#####################################################################################
# Copyright 2011 Normation SAS
#####################################################################################
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, Version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#####################################################################################

#######################################################
#
# Site specific promises
#
#######################################################

bundle common g
{
  vars:

    android::
      "rudder_base"                string => "/data/rudder";
      "rudder_var"                 string => "/data/rudder";
      "rudder_curl"                string => "/system/bin/curl";
      "rudder_rm"                  string => "/system/xbin/rm";

    !windows.!android::
      "rudder_base"                string => "/opt/rudder";
      "rudder_curl"                string => "/usr/bin/curl";
      "rudder_rm"                  string => "/bin/rm";

    solaris::
      "rudder_var"                 string => "/opt/rudder/var";

    !solaris.!windows.!android::
      "rudder_var"                 string => "/var/rudder";

    !windows::
      "rudder_bin"                 string => "${rudder_base}/bin";
      "rudder_sbin"                string => "${rudder_base}/sbin";
      "rudder_var_tmp"             string => "${rudder_var}/tmp"; # tmp generated data
      "rudder_base_sbin"           string => "${rudder_base}/sbin"; #folder where tools are installed
      "rudder_inventories"         string => "${rudder_var}/inventories";
      "uuid_file"                  string => "${rudder_base}/etc/uuid.hive";
      "rudder_disable_agent_file"  string => "${rudder_base}/etc/disable-agent";
      "rudder_tools"               string => "${rudder_var}/tools";
      "rudder_ncf"                 string => "${rudder_var}/ncf";
      "crontab"                    string => "/etc/crontab";

      # DEPRECATED: This variable is used in pre-2.9 Techniques.
      "rudder_dependencies"        string => "${rudder_var}/tools";

    windows::
      "rudder_base"                string => "${sys.winprogdir}\Rudder";
      "rudder_bin"                 string => "${rudder_base}\bin";
      "rudder_sbin"                string => "${rudder_base}\sbin";
      "rudder_var"                 string => "${sys.winprogdir}\Rudder\var";
      "rudder_var_tmp"             string => "${rudder_var}\tmp"; # tmp generated data
      "rudder_base_sbin"           string => "${rudder_base}\sbin"; #folder where tools are installed
      "rudder_inventories"         string => "${rudder_var}\inventories";
      "rudder_base_sbin_arg"       string => "${sys.winprogdir}\Rudder\sbin"; # for the installer command line
      "rudder_tools"               string => "${rudder_sbin}";
      "rudder_ncf"                 string => "${rudder_base}\ncf";
      "escaped_workdir"            string => escape("${sys.workdir}");
      "rudder_curl"                string => "\"${rudder_base_sbin}\curl\curl.exe\"";
      "uuid_file"                  string => "${rudder_base}\etc\uuid.hive";
      "rudder_disable_agent_file"  string => "${rudder_base}\etc\disable-agent";

      # DEPRECATED: This variable is used in pre-2.9 Techniques.
      "rudder_dependencies"        string => "${rudder_sbin}";

    any::
      "uuid"                        string => readfile("${g.uuid_file}", 60);
      "server_shares_folder"        string => "/var/rudder/share/${uuid}/promises/shares";
      "rudder_var_reports"          string => "${rudder_var}/reports";
      "davuser"                     string => "&DAVUSER&";
      "davpw"                       string => "&DAVPASSWORD&";
      "minicurl"                    string => "${rudder_bin}/rudder-perl ${sys.workdir}/inputs/common/utilities/minicurl";
      "excludedreps"                slist  => { "\.X11", ".*kde.*", "\.svn", "perl" };
      "rudder_tools_origin"         string => "/var/rudder/tools";
      "rudder_ncf_origin_common"    string => "/usr/share/ncf/tree";
      "rudder_ncf_origin_local"     string => "/var/rudder/configuration-repository/ncf";

      # DEPRECATED: This variable is used in pre-2.9 Techniques.
      "rudder_dependencies_origin"  string => "/var/rudder/tools";

    # The time at which the execution started
    (linux|cygwin).!(centos_3|redhat_3|centos_4|redhat_4)::
      "execRun"                    string => execresult("/bin/date --rfc-3339=second", "noshell");
    (linux|cygwin).(centos_3|redhat_3|centos_4|redhat_4)::
      # We would like to use date's "--rfc-3339=second" option here, but it is not available on older OSes (RHEL 3/4, AIX 5...)
      "execRun"                    string => execresult("/bin/date -u \"+%Y-%m-%d %T+00:00\"", "noshell");
    windows.!cygwin::
      "execRun"                    string => execresult("\"${g.rudder_sbin}\getDate.bat\"", "noshell");
    android::
      "execRun"                    string => execresult("/system/xbin/date \"+%Y-%m-%d %T%z\" | sed 's/\([-+][0-9][0-9]\)\([0-9][0-9]\)$/\1:\2/'", "useshell");
    aix::
      # AIX's date command doesn't have a "%z" option, so we fake it by using UTC
      "execRun"                    string => execresult("/bin/date -u \"+%Y-%m-%d %T+00:00\"", "noshell");
    !linux.!cygwin.!windows.!android.!aix::
      "execRun"                    string => execresult("/bin/date \"+%Y-%m-%d %T%:z\"", "noshell");

  classes:

    "curl_installed"           expression => isexecutable("${rudder_curl}");

}
